@@ -1,11 +1,8 @@
 //{{NO_DEPENDENCIES}}
-// Microsoft Developer Studio generated include file.
+// Microsoft Visual C++ generated include file.
 // Used by resource.rc
 //
-#define IDR_MENU2                       104
 #define IDI_ICON3                       105
-#define ID_FILE_CONFIG                  40003
-#define ID_FILE_EXIT                    40004
 
 // Next default values for new objects
 // 
