@@ -1,23 +1,25 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
+#ifndef GKRELLMD_H
+#define GKRELLMD_H
 
 #include <glib.h>
 
@@ -49,12 +51,9 @@
 #include <sys/select.h>
 #include <sys/wait.h>
 #else
-#define SIGTSTP SIGTERM
 #define ioctl ioctlsocket
-#include "../src/win32.h"
-#if defined(_WINDOWS)
-#include "resource.h"
-#endif
+	#include <winsock2.h>
+	#include <ws2tcpip.h>
 #endif
 
 #include <sys/stat.h>
@@ -92,8 +91,8 @@
 /* -------------------------------------------------------------------
 */
 #define GKRELLMD_VERSION_MAJOR   2
-#define GKRELLMD_VERSION_MINOR   2
-#define GKRELLMD_VERSION_REV     10
+#define GKRELLMD_VERSION_MINOR   3
+#define GKRELLMD_VERSION_REV     1
 #define GKRELLMD_EXTRAVERSION    ""
 
 #define GKRELLMD_CHECK_VERSION(major,minor,rev)    \
@@ -223,7 +222,8 @@ gint		gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
   /* Misc
   */
 void		gkrellmd_add_mailbox(gchar *);
-
+GkrellmdTicks *gkrellmd_ticks(void);
+gint		gkrellmd_get_timer_ticks(void);
 
 
 #if !GLIB_CHECK_VERSION(2,0,0)
@@ -251,3 +251,4 @@ gchar		*g_path_get_basename(gchar *file_name);
 
 #endif
 
+#endif // GKRELLMD_H
