@@ -1,30 +1,35 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellmd.h"
 #include "gkrellmd-private.h"
 
-#include "../src/inet.h"
+#ifdef WIN32
+	#include "win32-gui.h"
+#endif
 
-#ifndef HAVE_GETADDRINFO
+//#include "../src/inet.h"
+
+// we do have addrinfo on win32 but do not have getaddrinfo(), doh
+#if !defined(HAVE_GETADDRINFO) && !defined(WIN32)
 struct addrinfo
 	{
 	int		ai_flags;		/* AI_PASSIVE, AI_CANONNAME, AI_NUMERICHOST */
@@ -36,7 +41,7 @@ struct addrinfo
 	struct	sockaddr *ai_addr;	/* binary address */
 	struct	addrinfo *ai_next;	/* next structure in linked list */
 	};
-#endif
+#endif // HAVE_GETADDRINFO && !WIN32
 
 #if !defined(IPV6_V6ONLY) && defined(IPV6_BINDV6ONLY)
 #define	IPV6_V6ONLY	IPV6_BINDV6ONLY
@@ -202,8 +207,11 @@ cidr_match(struct sockaddr *sa, socklen_t salen, char *allowed)
 	char		*buf;
 	char		*p, *ep;
 	guchar		*addr, *pat;
-	u_int32_t	mask;
-	int			plen, i;
+	uint32_t	mask;
+	int		plen;
+#if defined(INET6)
+	int		i;
+#endif
 	gboolean	result;
 
 	buf = g_strdup(allowed);
@@ -253,13 +261,13 @@ cidr_match(struct sockaddr *sa, socklen_t salen, char *allowed)
 				if (plen < 32)
 					{
 					mask = htonl(~(0xffffffff >> plen));
-					if ((*(u_int32_t *)&addr[i] & mask) !=
-					    (*(u_int32_t *)&pat[i] & mask))
+					if ((*(uint32_t *)&addr[i] & mask) !=
+					    (*(uint32_t *)&pat[i] & mask))
 						return FALSE;
 					break;
 					}
-				if (*(u_int32_t *)&addr[i] !=
-				    *(u_int32_t *)&pat[i])
+				if (*(uint32_t *)&addr[i] !=
+				    *(uint32_t *)&pat[i])
 					return FALSE;
 				i += 4;
 				plen -= 32;
@@ -274,8 +282,8 @@ cidr_match(struct sockaddr *sa, socklen_t salen, char *allowed)
 			addr = (guchar *)&((struct sockaddr_in *)sa)->sin_addr;
 			pat = (guchar *)&((struct sockaddr_in *)&ss)->sin_addr;
 			mask = htonl(~(0xffffffff >> plen));
-			if ((*(u_int32_t *)addr & mask) !=
-			    (*(u_int32_t *)pat & mask))
+			if ((*(uint32_t *)addr & mask) !=
+			    (*(uint32_t *)pat & mask))
 				return FALSE;
 			break;
 		default:
@@ -454,6 +462,8 @@ parse_config(gchar *config, gchar *arg)
 		_GK.update_HZ = atoi(arg);
 	else if (!strcmp(config, "port") || !strcmp(config, "P"))
 		_GK.server_port = atoi(arg);
+	else if (!strcmp(config, "address") || !strcmp(config, "A"))
+		_GK.server_address = g_strdup(arg);
 	else if (!strcmp(config, "max-clients") || !strcmp(config, "m"))
 		_GK.max_clients = atoi(arg);
 	else if (!strcmp(config, "allow-host") || !strcmp(config, "a"))
@@ -628,6 +638,7 @@ usage(void)
 	printf(_("options:\n"));
 	printf(_("   -u, --update-hz F         Monitor update frequency\n"));
 	printf(_("   -m, --max-clients N       Number of simultaneous clients\n"));
+	printf(_("   -A, --address A           Address of network interface to listen on\n"));
 	printf(_("   -P, --port P              Server port to listen on\n"));
 	printf(_("   -a, --allow-host host     Allow connections from specified hosts\n"));
 	printf(_("   -c, --clear-hosts         Clears the current list of allowed hosts\n"));
@@ -644,6 +655,7 @@ usage(void)
 	printf(_("   -pe, --plugin-enable name Enable an installed plugin\n"));
 	printf(_("       --plist               List plugins and exit\n"));
 	printf(_("       --plog                Print plugin install log\n"));
+	printf(_("       --pidfile path        Create a PID file\n"));
 	printf(_("   -V, --verbose\n"));
 	printf(_("   -v, --version\n"));
 	}
@@ -728,7 +740,14 @@ socksetup(int af)
 	hints.ai_flags = AI_PASSIVE;
 	hints.ai_family = af;
 	snprintf(portnumber, sizeof(portnumber), "%d", _GK.server_port);
-	error = getaddrinfo(NULL, portnumber, &hints, &res);
+	if (!_GK.server_address || strlen(_GK.server_address) == 0)
+		{
+		error = getaddrinfo(NULL, portnumber, &hints, &res);
+		}
+	else
+		{
+		error = getaddrinfo(_GK.server_address, portnumber, &hints, &res);
+		}
 	if (error)
 		{
 		fprintf(stderr, "gkrellmd %s\n", gai_strerror(error));
@@ -743,7 +762,14 @@ socksetup(int af)
 	hints.ai_next = NULL;
 	hints.ai_addr = (struct sockaddr *) &sin;
 	sin.sin_family = PF_INET;
-	sin.sin_addr.s_addr = INADDR_ANY;
+	if (!_GK.server_address || strlen(_GK.server_address) == 0)
+		{
+		sin.sin_addr.s_addr = INADDR_ANY;
+		}
+	else
+		{
+		sin.sin_addr.s_addr = inet_addr(_GK.server_address);
+		}
 	sin.sin_port = htons(_GK.server_port);
 	res = &hints;
 #endif
@@ -771,8 +797,11 @@ socksetup(int af)
 		*/
 		if (1)
 			{
+#ifdef WIN32
+			const char on = 1;
+#else
 			const int on = 1;
-
+#endif
 			if (setsockopt(*s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0)
 				{
 				fprintf(stderr, "gkrellmd setsockopt (SO_REUSEADDR) failed\n");
@@ -899,11 +928,9 @@ drop_privileges(void)
 	}
 
 
-#if defined(WIN32) && defined(_WINDOWS)
-int APIENTRY WinMain(HINSTANCE	hInstance,
-					HINSTANCE	hPrevInstance,
-					LPSTR		lpCmdLine,
-					int			nCmdShow)
+#if defined(WIN32)
+int APIENTRY
+WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmd, int nCmdShow)
 	{
 	int		argc;
 	gchar	**argv;
@@ -913,7 +940,7 @@ int APIENTRY WinMain(HINSTANCE	hInstance,
 gint
 main(gint argc, gchar **argv)
 	{
-#endif
+#endif // WIN32
 #ifdef HAVE_GETADDRINFO
 	struct sockaddr_storage client_addr;
 #else
@@ -923,18 +950,23 @@ main(gint argc, gchar **argv)
 	struct timeval		tv;
 	GkrellmdClient		*client;
 	size_t				addr_len;
-	gint				fd, server_fd, client_fd, nbytes, i;
+	gint				fd, server_fd, client_fd, i;
+#if defined(WIN32)
+	gulong			nbytes;
+#else
+	gint				nbytes;
+#endif // WIN32
 	gint				max_fd = -1;
 	gint				listen_fds = 0;
 	gint				interval, result;
 
-#if defined(WIN32) && defined(_WINDOWS)
-    setupServerWindow(hInstance);
+#if defined(WIN32)
+    createServerWindow(hInst);
 
-    // need to get exe too
+    // lpCmd does not contain the exe name, GetCommandLine() does :)
 	cmd = GetCommandLine();
 	g_shell_parse_argv(cmd, &argc, &argv, NULL);
-#endif
+#endif // WIN32
 
 #ifdef ENABLE_NLS
 #ifdef LOCALEDIR
@@ -957,9 +989,11 @@ main(gint argc, gchar **argv)
 	else
 		{
 		signal(SIGTERM, cb_sigterm);
+#if !defined(WIN32)
 		signal(SIGQUIT, cb_sigterm);
-		signal(SIGINT, cb_sigterm);
 		signal(SIGTSTP, SIG_IGN);
+#endif
+		signal(SIGINT, cb_sigterm);
 		}
 
 	make_pidfile();
@@ -1015,12 +1049,13 @@ main(gint argc, gchar **argv)
 
 	interval = 1000000 / _GK.update_HZ;
 
-#if defined(WIN32) && defined(_WINDOWS)
-    while (notDone == 0)
+#if defined(WIN32)
+	while (!done)
         {
             {
                 MSG msg;
-                while (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE)) {
+		while (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE))
+			{
                     TranslateMessage(&msg); 
                     DispatchMessage(&msg); 
                 }
@@ -1067,7 +1102,7 @@ main(gint argc, gchar **argv)
 				{
 				client_fd = accept(server_fd,
 						(struct sockaddr *) &client_addr,
-						(socklen_t *) &addr_len);
+						(socklen_t *) (void *)&addr_len);
 				if (client_fd == -1)
 					{
 					fprintf(stderr, "gkrellmd accept() failed: %s\n",
@@ -1105,5 +1140,22 @@ main(gint argc, gchar **argv)
 			}
 		gkrellmd_update_monitors();
 		}
+		
+#ifdef WIN32
+	deleteServerWindow();
+#endif
 	return 0;
+	} // main()/WinMain()
+
+
+GkrellmdTicks *
+gkrellmd_ticks(void)
+	{
+	return &GK;
+	}
+
+gint
+gkrellmd_get_timer_ticks(void)
+	{
+	return GK.timer_ticks;
 	}
