@@ -1,27 +1,27 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellmd.h"
 #include "gkrellmd-private.h"
-
+#include <inttypes.h>
 
 GList			*gkrellmd_monitor_list;
 
@@ -555,14 +555,14 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (!disk->subdisk_parent)
 			{
 			if (gkrellmd_check_client_version(mon, 2, 2, 7) && disk->virtual)
-				snprintf(buf, sizeof(buf), "%s virtual %llu %llu\n",
+				snprintf(buf, sizeof(buf), "%s virtual %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			else
-				snprintf(buf, sizeof(buf), "%s %llu %llu\n",
+				snprintf(buf, sizeof(buf), "%s %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			}
 		else if (mon->privat->client->feature_subdisk)
-			snprintf(buf, sizeof(buf), "%s %s %llu %llu\n",
+			snprintf(buf, sizeof(buf), "%s %s %" PRIu64 " %" PRIu64 "\n",
 						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
 		else
 			continue;
@@ -1233,7 +1233,7 @@ serve_mem_data(GkrellmdMonitor *mon, gboolean first_serve)
 	if (mem.mem_changed || first_serve)
 		{
 		gkrellmd_set_serve_name(mon, "mem");
-		snprintf(buf, sizeof(buf), "%llu %llu %llu %llu %llu %llu\n",
+		snprintf(buf, sizeof(buf), "%" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 "\n",
 				mem.total, mem.used, mem.free,
 				mem.shared, mem.buffers, mem.cached);
 		gkrellmd_serve_data(mon, buf);
@@ -1242,7 +1242,7 @@ serve_mem_data(GkrellmdMonitor *mon, gboolean first_serve)
 	if (mem.swap_changed || first_serve)
 		{
 		gkrellmd_set_serve_name(mon, "swap");
-		snprintf(buf, sizeof(buf), "%llu %llu %lu %lu\n",
+		snprintf(buf, sizeof(buf), "%" PRIu64 " %" PRIu64 " %lu %lu\n",
 				mem.swap_total, mem.swap_used,
 				mem.swap_in, mem.swap_out);
 		gkrellmd_serve_data(mon, buf);
@@ -1853,6 +1853,10 @@ gkrellm_sensors_set_group(gpointer sr, gint group)
 		sensor->group = group;
 	}
 
+void
+gkrellm_sensors_sysdep_option(gchar *keyword, gchar *label, void (*func)())
+	{
+	}
 
   /* A sensor within a type is uniquely identified by its id_name.
   |  A sysdep interface may additionally use any of the triple integer
