@@ -1,25 +1,24 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
-
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
