@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 
@@ -24,6 +24,8 @@
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
 
+#include <inttypes.h>
+
 #if !defined(WIN32)
 #include <sys/socket.h>
 #include <utime.h>
@@ -46,6 +48,10 @@
 #endif
 #endif
 
+#if defined(__DragonFly__)
+#define HAVE_GETADDRINFO	1
+#endif
+
 #if defined(__FreeBSD__)
 #if __FreeBSD_version >= 400000
 #define HAVE_GETADDRINFO	1
@@ -145,7 +151,7 @@ client_cpu_line_from_server(gchar *line)
 	gint	n;
 	guint64	user, nice, sys, idle;
 
-	sscanf(line, "%d %llu %llu %llu %llu", &n, &user, &nice, &sys, &idle);
+	sscanf(line, "%d %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64, &n, &user, &nice, &sys, &idle);
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (Cpu *) list->data;
@@ -406,7 +412,7 @@ client_net_line_from_server(gchar *line)
 	gchar		name[32];
 	guint64		rx, tx;
 
-	sscanf(line, "%31s %llu %llu", name, &rx, &tx);
+	sscanf(line, "%31s %" PRIu64 " %" PRIu64, name, &rx, &tx);
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
@@ -557,6 +563,8 @@ client_inet_line_from_server(gchar *line)
 #endif
 	gint		n, slen;
 
+	if (_GK.debug_level & DEBUG_INET)
+		g_print("inet server: %s\n", line);
 	if (*(line + 1) == '0')
 		{
 		n = sscanf(line + 3, "%x %127[^:]:%x", &tcp.local_port,
@@ -680,7 +688,7 @@ struct
 static void
 client_mem_line_from_server(gchar *line)
 	{
-	sscanf(line, "%llu %llu %llu %llu %llu %llu",
+	sscanf(line, "%" PRIu64 "%" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64,
 			&mem.total, &mem.used, &mem.free,
 			&mem.shared, &mem.buffers, &mem.cached);
 	}
@@ -688,7 +696,7 @@ client_mem_line_from_server(gchar *line)
 static void
 client_swap_line_from_server(gchar *line)
 	{
-	sscanf(line, "%llu %llu %lu %lu",
+	sscanf(line, "%" PRIu64 " %" PRIu64 " %lu %lu",
 			&mem.swap_total, &mem.swap_used,
 			&mem.swap_in, &mem.swap_out);
 	}
@@ -1870,7 +1878,9 @@ gkrellm_client_mode_connect(void)
 
 	/* Now switch to non blocking and set up a read handler.
 	*/
+#ifndef WIN32
 	fcntl(client_fd, F_SETFL, O_NONBLOCK);
+#endif
 	client_input_id = gdk_input_add(client_fd, GDK_INPUT_READ,
 					(GdkInputFunction) read_server_input, NULL);
 
