@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -155,9 +155,13 @@ gkrellm_theme_file_exists(char *name, gchar *subdir)
 			else
 				path = g_strdup_printf("%s/%s_%d%s", _GK.theme_path,
 						name, _GK.theme_alternative, image_type[i]);
+#ifdef WIN32
+			if (stat(path, &st) == 0 && S_ISREG(st.st_mode))
+#else
 			if (   stat(path, &st) == 0
 				&& (S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))
 			   )
+#endif
 				return path;
 			g_free(path);
 			path = NULL;
@@ -171,9 +175,13 @@ gkrellm_theme_file_exists(char *name, gchar *subdir)
 		else
 			path = g_strdup_printf("%s/%s%s", _GK.theme_path, name,
 					image_type[i]);
+#ifdef WIN32
+			if (stat(path, &st) == 0 && S_ISREG(st.st_mode))
+#else
 		if (   stat(path, &st) == 0
 			&& (S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))
 		   )
+#endif
 			return path;
 		g_free(path);
 		path = NULL;
@@ -411,7 +419,7 @@ assign_style_entry(GkrellmStyle *style, gchar *value, gint entry_flag)
 		{
 		if (strcmp(value, "center") == 0)
 			style->label_position = GKRELLM_LABEL_CENTER;
-		else if (isdigit(*value))
+		else if (isdigit((unsigned char)*value))
 			style->label_position = atoi(value);
 		else
 			style->label_position = GKRELLM_LABEL_NONE;
@@ -1283,7 +1291,7 @@ destroy_monitor_specific_piximages(void)
 		{
 		mon = (GkrellmMonitor *) list->data;
 		destroy_piximage(&mon->privat->top_spacer.piximage);
-		destroy_piximage(&mon->privat->bottom_spacer.piximage);	
+		destroy_piximage(&mon->privat->bottom_spacer.piximage);
 		}
 	}
 
@@ -1627,7 +1635,7 @@ gkrellm_get_gkrellmrc_string(gchar *string_name)
 	}
 
 static gboolean
-parse_monitor_config_keyword(gchar *line)
+parse_monitor_config_keyword(GkrellmMonitor *mon_only, gchar *line)
 	{
 	GList			*list;
 	GkrellmMonitor	*mon;
@@ -1644,6 +1652,8 @@ parse_monitor_config_keyword(gchar *line)
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmMonitor *) list->data;
+		if (mon_only && mon != mon_only)
+			continue;
 		if (!mon->config_keyword || strcmp(mon->config_keyword, keyword))
 			continue;
 		while (*line == ' ' || *line == '\t')
@@ -1733,7 +1743,7 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 			}
 		if (conf->value)
 			{
-			if (isdigit(*s) || *s == '-')
+			if (isdigit((unsigned char)*s) || *s == '-')
 				val = atoi(s);
 			else if (!strcmp(s, "on") || !strcmp(s, "true"))
 				val = 1;
@@ -2266,13 +2276,13 @@ struct	_config	user_config[] =
 
   /* The user_config is read twice.  Early to load _GK config values and then
   |  again when when building gkrellm for the first time to load monitor user
-  |  config values.
+  |  config values.  It's also read at plugin enables in case existing configs.
   */
 void
-gkrellm_load_user_config(gboolean monitor_values)
+gkrellm_load_user_config(GkrellmMonitor *mon_only, gboolean monitor_values)
 	{
 	FILE	*f;
-	gchar	*s, *ss, *config, *config_2_1_14 = NULL;
+	gchar	*s, *ss, *config;
 	gchar	buf[CFG_BUFSIZE];
 
 	if (!monitor_values)
@@ -2289,13 +2299,8 @@ gkrellm_load_user_config(gboolean monitor_values)
 	config = gkrellm_make_config_file_name(gkrellm_homedir(),
 				GKRELLM_USER_CONFIG);
 	f = fopen(config, "r");
+	g_free(config);
 
-	if (!f)		/* >= 2.1.15 configs are not backward compatible so there is */
-		{		/* a new config file name.  < 2.1.15 is forward compatible   */
-		config_2_1_14 = gkrellm_make_config_file_name(gkrellm_homedir(),
-					GKRELLM_2_1_14_CONFIG);
-		f = fopen(config_2_1_14, "r");
-		}
 	if (f)
 		{
 		while (fgets(buf, sizeof(buf), f))
@@ -2311,7 +2316,7 @@ gkrellm_load_user_config(gboolean monitor_values)
 			if (!strncmp(s, "float_factor ", 13))
 				sscanf(s + 13, "%f", &_GK.float_factor);
 			else if (monitor_values)
-				parse_monitor_config_keyword(s);
+				parse_monitor_config_keyword(mon_only, s);
 			else
 				parse_config_line(s, &user_config[0],
 						sizeof(user_config) / sizeof (struct _config));
@@ -2322,10 +2327,6 @@ gkrellm_load_user_config(gboolean monitor_values)
 		_GK.chart_width = CHART_WIDTH_MIN;
 	if (_GK.chart_width > CHART_WIDTH_MAX)
 		_GK.chart_width = CHART_WIDTH_MAX;
-//	if (config_2_1_14)
-//		rename(config_2_1_14, config);
-	g_free(config);
-	g_free(config_2_1_14);
 	}
 
 void
@@ -2405,24 +2406,28 @@ gkrellm_make_config_file_name(gchar *dir, gchar *config)
 void
 gkrellm_save_user_config(void)
 	{
-	FILE	*f;
-	GList	*list;
+	FILE			*f, *ff;
+	GList			*list;
 	GkrellmMonitor	*mon;
-	gint	i;
-	mode_t	mode;
-	gchar	*config;
+	gint			i;
+	mode_t			mode;
+	gchar			*config, *config_new;
 
-	if (_GK.demo)
+	if (_GK.demo || _GK.no_config)
 		return;
 	config = gkrellm_make_config_file_name(gkrellm_homedir(),
-								GKRELLM_USER_CONFIG);
-	f = fopen(config, "w");
-	g_free(config);
+					GKRELLM_USER_CONFIG);
+	config_new = g_strconcat(config, ".new", NULL);
+
+	f = fopen(config_new, "w");
 	if (f == NULL)
 		{
-		printf(_("Cannot open config file %s for writing.\n"), config);
+		printf(_("Cannot open config file %s for writing.\n"), config_new);
+		g_free(config_new);
+		g_free(config);
 		return;
 		}
+
 	fprintf(f,
 		"### GKrellM user config.  Auto written, do not edit (usually) ###\n");
 	fprintf(f, "### Version %d.%d.%d ###\n",
@@ -2447,6 +2452,57 @@ gkrellm_save_user_config(void)
 			gkrellm_record_state(INTERNAL, NULL);
 			}
 		}
+
+	if (   !_GK.config_clean
+	    && (ff = fopen(config, "r")) != NULL
+	   )
+		{
+		gchar		buf[CFG_BUFSIZE], *keyword, *ch, tmp;
+		struct _config
+					*cf,
+					*const end = user_config +
+								sizeof user_config / sizeof *user_config;
+
+		while (fgets(buf, sizeof(buf), ff))
+			{
+			for (ch = buf; *ch == ' ' || *ch == '\t'; ++ch)
+				;
+			if (*ch == '\n' || *ch == '#' || !*ch)
+				continue;
+			keyword = ch;
+			while (*ch && *ch != ' ' && *ch != '\t' && *ch != '\n')
+				++ch;
+			tmp = *ch;
+			*ch = 0;
+
+			for (list = gkrellm_monitor_list; list; list = list->next)
+				{
+				mon = (GkrellmMonitor *) list->data;
+				if (   mon->save_user_config && mon->privat->enabled
+				    && mon->config_keyword
+				    && !strcmp(mon->config_keyword, keyword)
+				   )
+					break;
+				}
+			if (list)
+				continue;
+
+			cf = user_config;
+			while (cf != end && strcmp(cf->option, keyword))
+				++cf;
+			if (cf != end)
+				continue;
+			if (!strcmp("float_factor", keyword))
+				continue;
+
+			*ch = tmp;
+			fputs(buf, f);
+			}
+		fclose(ff);
+		}
+	fclose(f);
+	rename(config_new, config);
+
 #if defined (S_IRUSR)
 	mode = (S_IRUSR | S_IWUSR);
 #elif defined (S_IREAD)
@@ -2454,9 +2510,10 @@ gkrellm_save_user_config(void)
 #else
 	mode = 0600;
 #endif
-	fchmod(fileno(f), mode);
-	fclose(f);
+	chmod(config, mode);
+
+	g_free(config);
+	g_free(config_new);
 
 	_GK.config_modified = FALSE;
 	}
-
