@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -1274,7 +1274,7 @@ create_cpu_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
 	GtkWidget	*button;
-	GtkWidget	*hbox, *hbox1, *vbox, *vbox1;
+	GtkWidget	*hbox, *vbox, *vbox1;
 	GtkWidget	*text;
 	GtkWidget	*table;
 	GSList		*group;
@@ -1312,8 +1312,9 @@ create_cpu_tab(GtkWidget *tab_vbox)
 				cb_config_tracking, NULL,
 		_("Apply any CPU chart config height change to all CPU charts"));
 
-		vbox1 = gkrellm_gtk_framed_vbox(vbox, _("SMP Charts Select"),
-				4, FALSE, 0, 2);
+		vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("SMP Charts Select"),
+				4, 0, TRUE);
 
 		button = gtk_radio_button_new_with_label(NULL, _("Real CPUs."));
 		gtk_box_pack_start(GTK_BOX(vbox1), button, TRUE, TRUE, 0);
@@ -1340,15 +1341,15 @@ create_cpu_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
-	vbox1 = gkrellm_gtk_framed_vbox(hbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox1 = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+
 	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox1), text_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, TRUE, TRUE, 0);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
@@ -1366,13 +1367,9 @@ create_cpu_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(hbox), vbox1, FALSE, TRUE, 4);
-	gkrellm_gtk_alert_button(vbox1, NULL, FALSE, FALSE, 4, FALSE,
-			cb_set_alert, NULL);
-
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, TRUE, 0,2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	vbox1 = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	table = gkrellm_gtk_launcher_table_new(vbox1, n_cpus);
@@ -1388,6 +1385,11 @@ create_cpu_tab(GtkWidget *tab_vbox)
 				G_CALLBACK(cb_launch_entry), cpu);
 		}
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gkrellm_gtk_alert_button(hbox, NULL, FALSE, FALSE, 4, TRUE,
+			cb_set_alert, NULL);
+
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
 	text = gkrellm_gtk_scrolled_text_view(vbox, NULL,
