@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -395,6 +395,8 @@ format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 				blocks = r_blocks;
 			else if (c == 'w')
 				blocks = w_blocks;
+			else if (c == 'L')
+				len = snprintf(buf, size, "%s", disk->name);
 			else if (c == 'H')
 				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else
@@ -493,7 +495,7 @@ gkrellm_disk_temperature_display(gpointer sr, gchar *id_name, gfloat t,
 		{
 		disk = (DiskMon *) list->data;
 		if (   strncmp(disk->name, disk_name, len)
-			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+			|| (disk->name[len] != '\0' && !isdigit((unsigned char)disk->name[len]))
 		   )
 			continue;
 		if (!disk->enabled || !disk->chart || !disk->chart->panel->decal_list)
@@ -554,7 +556,7 @@ gkrellm_disk_temperature_remove(gchar *id_name)
 		if (!disk->chart)
 			continue;
 		if (   strncmp(disk->name, disk_name, len)
-			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+			|| (disk->name[len] != '\0' && !isdigit((unsigned char)disk->name[len]))
 		   )
 			continue;
 		p = disk->chart->panel;
@@ -1319,6 +1321,7 @@ static gchar	*disk_info_text[] =
 {
 N_("<h>Chart Labels\n"),
 N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$L    the Disk label\n"),
 N_("\t$M    maximum chart value\n"),
 N_("\t$T    total read bytes + write bytes\n"),
 N_("\t$r    read bytes\n"),
@@ -1396,13 +1399,12 @@ create_disk_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
 	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 350, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo, FALSE, FALSE, 2);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
@@ -1418,8 +1420,9 @@ create_disk_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-						4, TRUE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox, NULL,
 						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_show(launch_vbox);
