@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
  
 #include "gkrellm.h"
@@ -154,6 +154,13 @@ _log_active_port(ActiveTCP *tcp)
 	tcp->new_hit = 1;
 	new_tcp = _tcp_alloc();
 	*new_tcp = *tcp;
+
+	if (_GK.debug_level & DEBUG_INET)
+		{
+		ap = inet_ntoa(tcp->remote_addr);
+		g_print("inet  OO----->  %x %s:%x\n",
+				tcp->local_port, ap, tcp->remote_port);
+		}
 	active_tcp_list = g_list_prepend(active_tcp_list, (gpointer) new_tcp);
 	return new_tcp;		/* A new hit	*/
 	}
@@ -305,6 +312,8 @@ format_chart_text(InetMon *in, gchar *src_string, gchar *buf, gint size)
 						value += gkrellm_get_chartdata_data(cd, w - n);
 					}
 				}
+			else if (c == 'H')
+				s1 = gkrellm_sys_get_host_name();
 			if (value >= 0)
 				len = snprintf(buf, size, "%d", value);
 			else
@@ -502,7 +511,7 @@ update_inet(void)
 	GkrellmChart	*cp;
 	ActiveTCP		*tcp;
 	GList			*list;
-	gchar			buf[32];
+	gchar			buf[32], *ap;
 	gint			i;
 	static gint		check_tcp;
 
@@ -536,6 +545,12 @@ update_inet(void)
 				if (list == active_tcp_list)
 					active_tcp_list = active_tcp_list->next;
 				list = g_list_remove(list, tcp);
+				if (_GK.debug_level & DEBUG_INET)
+					{
+					ap = inet_ntoa(tcp->remote_addr);
+					g_print("inet  XX----->  %x %s:%x\n",
+							tcp->local_port, ap, tcp->remote_port);
+					}
 				free_tcp_list = g_list_prepend(free_tcp_list, tcp);
 				}
 			else
@@ -633,6 +648,7 @@ get_connection_string_thread(void *data)
 	struct hostent			*hostent;
 	gchar					buf[64];
 #endif
+	gchar					*remote_host, *udp_note;
 
 	if (in->connection_string)
 		in->connection_string = g_string_truncate(in->connection_string, 0);
@@ -648,8 +664,7 @@ get_connection_string_thread(void *data)
 			case AF_INET:
 				sin = (struct sockaddr_in *)&ss;
 				salen = sizeof(struct sockaddr_in);
-				memcpy(&sin->sin_addr,
-						&tcp->remote_addr, salen);
+				memcpy(&sin->sin_addr, &tcp->remote_addr, salen);
 #if defined(SIN6_LEN)
 				sin->sin_len = salen;
 #endif
@@ -658,8 +673,7 @@ get_connection_string_thread(void *data)
 			case AF_INET6:
 				sin6 = (struct sockaddr_in6 *)&ss;
 				salen = sizeof(struct sockaddr_in6);
-				memcpy(&sin6->sin6_addr,
-						&tcp->remote_addr6, salen);
+				memcpy(&sin6->sin6_addr, &tcp->remote_addr6, salen);
 #if defined(SIN6_LEN)
 				sin6->sin6_len = salen;
 #endif
@@ -673,20 +687,19 @@ get_connection_string_thread(void *data)
 		if (getnameinfo((struct sockaddr *)&ss, salen,
 					hbuf, sizeof(hbuf), NULL, 0, flag))
 			continue;
-		snprintf(buf, sizeof(buf), "%6d:  %s\n",
-					tcp->local_port, hbuf);
+		remote_host = hbuf;
 #else
 		hostent = gethostbyaddr((char *) &tcp->remote_addr,
 					sizeof(struct in_addr), AF_INET);
 		if (hostent)
-			snprintf(buf, sizeof(buf),
-				 "%6d:  %s\n", tcp->local_port,
-				 hostent->h_name);
+			remote_host = hostent->h_name;
 		else
-			snprintf(buf, sizeof(buf),
-				 "%6d:  %s\n", tcp->local_port,
-				 inet_ntoa(tcp->remote_addr));
+			remote_host = inet_ntoa(tcp->remote_addr);
 #endif
+		udp_note = tcp->is_udp ? " (UDP)" : "";
+		snprintf(buf, sizeof(buf), "%6d:  %s%s\n",
+				tcp->local_port, remote_host, udp_note);
+
 		g_string_append(in->connection_string, buf);
 		}
 	if (in->connection_string->len == 0)
@@ -1067,7 +1080,7 @@ save_inet_config(FILE *f)
 		}
 	fprintf(f, "%s text_format all %s\n", INET_CONFIG_KEYWORD, text_format);
 	if (!_GK.client_mode)
-		fprintf(f, "%s update_interval %d\n",
+		fprintf(f, "%s update_interval all %d\n",
 					INET_CONFIG_KEYWORD, update_interval);
 	}
 
@@ -1986,13 +1999,14 @@ create_inet_tab(GtkWidget *tab_vbox)
             		cb_update_interval, NULL, FALSE,
             		_("Seconds between updates"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-				4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	label = gtk_label_new("");	/* padding */
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
 	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 350, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo, FALSE, FALSE, 2);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
@@ -2008,8 +2022,9 @@ create_inet_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, TRUE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_show(launch_vbox);
