@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2005 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -192,9 +192,10 @@ force_meminfo_update(void)
 static gint
 format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 	{
-	gulong	t, u, f;
-	gchar	*s, *label;
-	gint	len;
+	gulong		t, u, f, ur, fr;
+	gchar		*s, *label;
+	gint		len;
+	gboolean	raw;
 
 	if (!buf || size < 1)
 		return -1;
@@ -207,11 +208,17 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 	u = MEG(mm->used);
 	f = t - u;
 
+	fr = MEG(mem.free);
+	ur = u + MEG(mem.shared + mem.buffers + mem.cached);
+
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
+		raw = FALSE;
 		if (*s == '$' && *(s + 1) != '\0')
 			{
+			if (*(s + 2) == 'r')  /* print raw free/used */
+				raw = TRUE;
 			switch(*(s + 1))
 				{
 				case 'l':
@@ -221,18 +228,20 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 					len = snprintf(buf, size, "%ldM", t);
 					break;
 				case 'u':
-					len = snprintf(buf, size, "%ldM", u);
+					len = snprintf(buf, size, "%ldM", raw ? ur : u);
 					break;
 				case 'U':
 					if (t > 0)
-						len = snprintf(buf, size, "%ld%%", 100 * u / t);
+						len = snprintf(buf, size, "%ld%%",
+							100 * (raw ? ur : u) / t);
 					break;
 				case 'f':
-					len = snprintf(buf, size, "%ldM", f);
+					len = snprintf(buf, size, "%ldM", raw ? fr : f);
 					break;
 				case 'F':
 					if (t > 0)
-						len = snprintf(buf, size, "%ld%%", 100 * f / t);
+						len = snprintf(buf, size, "%ld%%",
+							100 * (raw ? fr : f) / t);
 					break;
 				case 's':
 					if (mm == &mem)
@@ -260,6 +269,8 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 					break;
 				}
 			++s;
+			if (raw)
+				++s;
 			}
 		else
 			*buf = *s;
@@ -1246,6 +1257,9 @@ N_("\t$s    shared MiB\n"),
 N_("\t$b    buffered MiB\n"),
 N_("\t$c    cached MiB\n"),
 "\n",
+N_("The free and used variables may have a 'r' qualifier for printing\n"
+   "raw free and raw used values.  For example: $fr for raw free.\n"),
+"\n",
 N_("Substitution variables may be used in alert commands.\n"),
 
 "\n",
@@ -1275,7 +1289,9 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 /* --Options Tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Swap"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Swap"),
+				4, 0, TRUE);
     gkrellm_gtk_check_button_connected(vbox1, NULL,
 				swap_chart.enabled, FALSE, FALSE, 0,
 				cb_swap_chart_enable, NULL,
@@ -1291,7 +1307,9 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	if (!swap.enabled)
 		gtk_widget_set_sensitive(swap_alert_button, FALSE);
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Memory"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Memory"),
+				4, 0, TRUE);
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
     gkrellm_gtk_check_button_connected(hbox, NULL,
@@ -1310,13 +1328,11 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
 	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo, FALSE, FALSE, 2);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
@@ -1334,12 +1350,13 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Panel Labels"),
-			4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Panel Labels"),
+				4, 6, TRUE);
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	mem_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), mem_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), mem_format_combo, TRUE, TRUE, 0);
 	list = NULL;
 	list = g_list_append(list, mem.data_format);
 	list = g_list_append(list, DEFAULT_FORMAT);
@@ -1356,10 +1373,12 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	label = gtk_label_new(_("Mem"));
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	swap_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), swap_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), swap_format_combo, TRUE, TRUE, 0);
 	list = NULL;
 	list = g_list_append(list, swap.data_format);
 	list = g_list_append(list, DEFAULT_FORMAT);
@@ -1375,10 +1394,11 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	label = gtk_label_new(_("Swap"));
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	table = gkrellm_gtk_launcher_table_new(vbox1, 2);
 	gkrellm_gtk_config_launcher(table, 0,  &mem_launch_entry,
 				&mem_tooltip_entry, _("Mem"), &(mem.launch));
