@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -81,7 +81,9 @@ typedef struct
 	GkrellmChartconfig	*chart_config;
 	gboolean			enabled,
 						chart_labels,
-						force_up;
+						force_up,
+						real;
+
 	gchar				*label;
 
 	GkrellmAlert		*alert;
@@ -159,7 +161,7 @@ strcmp_net_name(NetMon *net1, NetMon *net2)
 	gint	n, n1, n2, len;
 
 	for (s = net1->name; *s; ++s)
-		if (isdigit(*s))
+		if (isdigit((unsigned char)*s))
 			break;
 	if (!*s)
 		return strcmp(net1->name, net2->name);
@@ -291,6 +293,7 @@ gkrellm_net_routed_event(gchar *name, gboolean routed)
 	else
 		net->down_event = TRUE;
 	net->up = routed;
+	net->real = TRUE;
 	}
 
 void
@@ -304,6 +307,7 @@ gkrellm_net_assign_data(gchar *name, gulong rx, gulong tx)
 		net->up = TRUE;
 	net->rx = rx;
 	net->tx = tx;
+	net->real = TRUE;
 	}
 
 void
@@ -1258,6 +1262,9 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 				len = snprintf(buf, size, "%s", net->name);
 			else if (c == 'H')
 				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
+			else if ((s1 = gkrellm_plugin_get_exported_label(mon_net, c,
+						net->name)) != NULL)
+				len = snprintf(buf, size, "%s", s1);
 			else
 				{
 				*buf = *s;
@@ -2414,6 +2421,8 @@ save_net_config(FILE *f)
 	for (list = net_mon_list; list; list = list->next)
 		{
 		net = (NetMon *) list->data;
+		if (!net->enabled  && !net->real)
+			continue;
 		fprintf(f, "%s enables %s %d %d %d\n", NET_CONFIG_KEYWORD, net->name,
 				net->enabled, net->chart_labels, net->force_up);
 		gkrellm_save_chartconfig(f, net->chart_config,
@@ -3030,8 +3039,9 @@ create_net_tab(GtkWidget *tab_vbox)
 		g_signal_connect(G_OBJECT(net->label_entry), "changed",
 					G_CALLBACK(cb_label_entry), net);
 
-		vbox = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-						4, FALSE, 0, 2);
+		vbox = gkrellm_gtk_category_vbox(vbox,
+					_("Launch Commands"),
+					4, 0, TRUE);
 		table = gkrellm_gtk_launcher_table_new(vbox, 1);
 		gkrellm_gtk_config_launcher(table, 0,  &net->launch_entry,
 						&net->tooltip_entry, net->name, &net->launch);
@@ -3043,13 +3053,13 @@ create_net_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
 	text_format_combo = gtk_combo_new();
 	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo, FALSE, FALSE, 2);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
