@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #include "gkrellm.h"
@@ -164,7 +164,7 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 
 	sensor->default_factor = factor;
 	sensor->factor
-			= (sensor->default_factor > 0.0 ? sensor->default_factor : 1.0);
+			= (sensor->default_factor != 0.0 ? sensor->default_factor : 1.0);
 	sensor->default_offset = sensor->offset = offset;
 	sensor->type = type;
 	sensor->id = id;
@@ -447,11 +447,11 @@ cb_command_process(GkrellmAlert *alert, gchar *src, gchar *buf, gint size,
 					fmt = "%.0f";
 				else if (sensor->type == SENSOR_TEMPERATURE)
 					fmt = "%.1f";
-				else
+				else				/* SENSOR_VOLTAGE */
 					fmt = "%.2f";
 				len = snprintf(buf, size, fmt, sensor->value);
 				}
-			else if (c == 'l')
+			else if (c == 'l' || c == 'L')
 				len = snprintf(buf, size, "%s", sensor->name_locale);
 			++s;
 			}
@@ -504,7 +504,7 @@ sensor_read_temperature(Sensor *sensor, gfloat *temp, gchar *units)
 		}
 	if (! found_temp && _GK.demo)
 		{
-		t = 90.0 + (gfloat)(random() & 0xf);
+		t = 90.0 + (gfloat)(rand() & 0xf);
 		found_temp = TRUE;
 		}
 	if (temp)
@@ -542,7 +542,7 @@ sensor_read_fan(Sensor *sensor, gfloat *fan)
 		}
 	if (! found_fan && _GK.demo)
 		{
-		f = 4980 + (gfloat)(random() & 0x3f);
+		f = 4980 + (gfloat)(rand() & 0x3f);
 		found_fan = TRUE;
 		}
 	if (fan)
@@ -582,7 +582,7 @@ sensor_read_voltage(Sensor *sensor, gfloat *voltage)
 		}
 	if (! found_voltage && _GK.demo)
 		{
-		v = 2.9 + (gfloat)(random() & 0x7) * 0.1;
+		v = 2.9 + (gfloat)(rand() & 0x7) * 0.1;
 		found_voltage = TRUE;
 		}
 	if (voltage)
@@ -739,7 +739,7 @@ sensor_reset_location(Sensor *sr)
 	}
 
 void
-gkrellm_sensors_interface_remove(gint interface)
+gkrellm_sensors_interface_remove(gint _interface)
 	{
 	GList		*list;
 	Sensor		*sensor;
@@ -751,7 +751,7 @@ gkrellm_sensors_interface_remove(gint interface)
 		for (list = sensor_list; list; list = list->next)
 			{
 			sensor = (Sensor *) list->data;
-			if (sensor->inter == interface)
+			if (sensor->inter == _interface)
 				{
 				sensor_reset_location(sensor);
 				g_free(sensor->id_name);
@@ -1650,8 +1650,19 @@ gkrellm_init_sensor_monitor(void)
 #define	SENSOR_CONFIG_FILE			"sensor-config"
 #define	SENSOR_2_1_14_CONFIG_FILE	"sensors_config"
 
+typedef struct
+	{
+	gchar		*config_keyword,
+				*config_label;
+	gboolean	value;
+	void		(*func)(gboolean value);
+	}
+	SysdepOption;
+
 static void		cb_alert_config(GkrellmAlert *ap, Sensor *sr);
 
+static GList	*sysdep_option_list;
+
 static void
 create_sensor_alert(Sensor *s)
 	{
@@ -1675,16 +1686,29 @@ create_sensor_alert(Sensor *s)
 	gkrellm_alert_config_connect(s->alert, cb_alert_config, s);
 	}
 
+void
+gkrellm_sensors_sysdep_option(gchar *keyword, gchar *label, void (*func)())
+	{
+	SysdepOption	*so = NULL;
+
+	so = g_new0(SysdepOption, 1);
+	sysdep_option_list = g_list_append(sysdep_option_list, so);
+	so->config_keyword = g_strdup(keyword);
+	so->config_label = g_strdup(label);
+	so->func = func;
+	}
+
 static void
 save_sensors_config(FILE *f_not_used)
 	{
-	FILE	*f;
-	GList	*list;
-	Sensor	*s;
-	gchar	*config, quoted_name[128], buf[128];
-	gfloat	factor, offset;
+	FILE			*f;
+	GList			*list;
+	Sensor			*s;
+	SysdepOption	*so;
+	gchar			*config, quoted_name[128], buf[128];
+	gfloat			factor, offset;
 
-	if (!sensor_list)
+	if (!sensor_list || _GK.no_config)
 		return;
 	sprintf(buf, "%s/%s", GKRELLM_DIR, SENSOR_CONFIG_FILE);
 	config = gkrellm_make_config_file_name(gkrellm_homedir(), buf);
@@ -1701,14 +1725,21 @@ save_sensors_config(FILE *f_not_used)
 				SENSOR_CONFIG_KEYWORD, SENSOR_FLOAT_FACTOR);
 	fprintf(f, "%s gkrellm_float_factor %.0f\n",
 				SENSOR_CONFIG_KEYWORD, GKRELLM_FLOAT_FACTOR);
+	for (list = sysdep_option_list; list; list = list->next)
+		{
+		so = (SysdepOption *) list->data;
+		fprintf(f, "%s sysdep_option %s %d\n", SENSOR_CONFIG_KEYWORD,
+					so->config_keyword, so->value);
+		}
+
 	for (list = sensor_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
 		if (s->name && *(s->name))
 			{
 			snprintf(quoted_name, sizeof(quoted_name), "\"%s\"", s->id_name);
-			factor = (s->default_factor > 0.0 ? s->factor : 0.0);
-			offset = (s->default_factor > 0.0 ? s->offset : 0.0);
+			factor = (s->default_factor != 0.0 ? s->factor : 0.0);
+			offset = (s->default_factor != 0.0 ? s->offset : 0.0);
 			fprintf(f, "%s \"%s\" %s %.0f %.0f %d %d\n",
 				SENSOR_CONFIG_KEYWORD,
 				s->name, quoted_name,
@@ -1727,6 +1758,7 @@ save_sensors_config(FILE *f_not_used)
 			fprintf(f, "%s vref \"%s\" \"%s\"\n", SENSOR_CONFIG_KEYWORD,
 					s->id_name, s->vref->id_name);
 		}
+
 	fprintf(f, "%s units_fahrenheit %d\n", SENSOR_CONFIG_KEYWORD,
 				units_fahrenheit);
 	fprintf(f, "%s volt_display_mode %d\n", SENSOR_CONFIG_KEYWORD,
@@ -1740,15 +1772,17 @@ save_sensors_config(FILE *f_not_used)
 static void
 load_sensors_config(gchar *arg)
 	{
-	Sensor		*s;
-	gchar		config[32], item[CFG_BUFSIZE], item1[CFG_BUFSIZE];
-	gchar		label[64], id_name[CFG_BUFSIZE];
-	gint		n;
-	gfloat		f	= 1.0,
-				o	= 0.0;
-	gint		e	= 0,
-				location = 0;
-	gfloat		save_factor;
+	Sensor			*s;
+	SysdepOption	*so;
+	GList			*list;
+	gchar			config[32], item[CFG_BUFSIZE], item1[CFG_BUFSIZE];
+	gchar			label[64], id_name[CFG_BUFSIZE];
+	gint			n;
+	gfloat			f	= 1.0,
+					o	= 0.0;
+	gint			e	= 0,
+					location = 0;
+	gfloat			save_factor;
 
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n != 2)
@@ -1774,6 +1808,25 @@ load_sensors_config(gchar *arg)
 		   )
 			s->vref = lookup_sensor_from_id_name(item1);
 		}
+	else if (!strcmp(config, "sysdep_option"))
+		{
+		if (sscanf(item, "%63s %[^\n]", id_name, item1) == 2)
+			{
+			so = NULL;
+			for (list = sysdep_option_list; list; list = list->next)
+				{
+				so = (SysdepOption *) list->data;
+				if (!strcmp(so->config_keyword, id_name))
+					break;
+				so = NULL;
+				}
+			if (so && so->func)
+				{
+				so->value = atoi(item1);
+				(*so->func)(so->value);
+				}
+			}
+		}
 	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
 		{
 		if (   sscanf(item, "\"%63[^\"]\" %[^\n]", id_name, item1) == 2
@@ -1797,7 +1850,7 @@ load_sensors_config(gchar *arg)
 			 && (s = map_sensor_label(label, id_name)) != NULL
 			)
 		{
-		if (f > 0.0 && s->default_factor > 0.0)
+		if (f != 0.0 && s->default_factor != 0.0)
 			{
 			s->factor = f / sensor_float_factor;
 			s->offset = o / sensor_float_factor;
@@ -2496,6 +2549,14 @@ fix_temp_alert(Sensor *s)
 	gkrellm_alert_window_destroy(&s->alert);
 	}
 
+static void
+sysdep_option_cb(GtkWidget *button, SysdepOption *so)
+	{
+	if (!so)
+		return;
+	so->value = GTK_TOGGLE_BUTTON(button)->active;
+	}
+
 static void
 cb_temperature_units(GtkWidget *button, gpointer data)
 	{
@@ -2549,7 +2610,7 @@ sensors_apply(void)
 	if (mbmon_port_entry)
 		{
 		str = gkrellm_gtk_entry_get_text(&mbmon_port_entry);
-		if (isdigit(*str))
+		if (isdigit((unsigned char)*str))
 			{
 			port = atoi(str);
 			if (_GK.mbmon_port != port)
@@ -2617,6 +2678,8 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	GtkWidget				*label, *entry;
 	GtkTreeModel			*model;
 	GtkCellRenderer			*renderer;
+	GList					*list;
+	SysdepOption			*so;
 	gchar					buf[32];
 	gint					i;
 
@@ -2759,6 +2822,20 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	if (!sensor_list)
 		gtk_widget_set_sensitive(button, FALSE);
 
+	if (sysdep_option_list)
+		{
+		box = gkrellm_gtk_category_vbox(vbox, _("Options"), 4, 0, TRUE);
+		for (list = sysdep_option_list; list; list = list->next)
+			{
+			so= (SysdepOption *) list->data;
+			if (so->config_label)
+				gkrellm_gtk_check_button_connected(box, NULL,
+						so->value, FALSE, FALSE, 0,
+						sysdep_option_cb, so,
+						so->config_label);
+			}
+		}
+
 	button = display_mode_button[display_mode];
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
