@@ -1,25 +1,26 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  BSD code: Hajimu UMEMOTO <ume@FreeBSD.org>
+|  bsd-common.c code is Copyright (C):
+|            Hajimu UMEMOTO <ume@FreeBSD.org>
 |            Anthony Mallet <anthony.mallet@useless-ficus.net>
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 
 #if !(defined(__FreeBSD__) && __FreeBSD_version < 410000) && \
@@ -255,7 +256,11 @@ void
 gkrellm_sys_fs_get_mounts_list(void)
 	{
 	gchar		*s, *dev, *dir, *type;
+#if (defined(__NetBSD__) && __NetBSD_Version__ >= 299000900) /* NetBSD 2.99.9 */
+	struct statvfs	*mntbuf;
+#else
 	struct statfs	*mntbuf;
+#endif
 	gint		mntsize, i;
 
 	if ((mntsize = getmntinfo(&mntbuf, MNT_NOWAIT)) == 0)
@@ -281,12 +286,24 @@ gkrellm_sys_fs_get_mounts_list(void)
 void
 gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
 	{
+#if (defined(__NetBSD__) && __NetBSD_Version__ >= 299000900) /* NetBSD 2.99.9 */
+	struct statvfs	st;
+#else
 	struct statfs	st;
+#endif
 
+#if (defined(__NetBSD__) && __NetBSD_Version__ >= 299000900) /* NetBSD 2.99.9 */
+	if (!statvfs(dir, &st))
+#else
 	if (!statfs(dir, &st))
+#endif
 		gkrellm_fs_assign_fsusage_data(fs,
 					(glong) st.f_blocks, (glong) st.f_bavail,
+#if (defined(__NetBSD__) && __NetBSD_Version__ >= 299000900) /* NetBSD 2.99.9 */
+					(glong) st.f_bfree, (glong) st.f_frsize);
+#else
 					(glong) st.f_bfree, (glong) st.f_bsize);
+#endif
 	else
 		gkrellm_fs_assign_fsusage_data(fs, 0, 0, 0, 0);
 	}
@@ -345,7 +362,7 @@ gkrellm_sys_fs_init(void)
 	}
 #endif
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__DragonFly__)
 gboolean
 gkrellm_sys_fs_init(void)
 	{
