@@ -1,22 +1,22 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2007 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
 */
 #include <limits.h>
 #include <errno.h>
@@ -25,6 +25,8 @@
 #include <dirent.h>
 #include <string.h>
 
+#include <inttypes.h>
+
 
 static gboolean		need_locale_fix,
 					have_diskstats,
@@ -152,7 +154,8 @@ linux_read_proc_stat(void)
 		{
 		if (buf[0] == 'c' && buf[1] == 'p')
 			{
-			n = sscanf(buf, "%*s %Lu %Lu %Lu %Lu %Lu",
+			n = sscanf(buf,
+				"%*s %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64,
 						&user, &nice, &sys, &idle, &iowait);
 
 			if (n == 5)			/* iowait is new in kernel 2.5	*/
@@ -331,6 +334,9 @@ gkrellm_sys_cpu_init(void)
 #if !defined(NBD_MAJOR)
 #define NBD_MAJOR 43
 #endif
+#if !defined(I2O_MAJOR)
+#define I2O_MAJOR 80
+#endif
 
 
 struct _disk_name_map
@@ -361,6 +367,7 @@ static struct _disk_name_map
 	{"scd",	SCSI_CDROM_MAJOR,			16,	'0' },	/* 11: scd0-scd16 */
 	{"sr",	SCSI_CDROM_MAJOR,			16,	'0' },	/* 11: sr0-sr16 */
 	{"md",	MD_MAJOR,					0,	'0' },	/* 9:  md0-md3 */
+	{"i2o/hd", I2O_MAJOR,				16,	'a' },	/* 80:  i2o/hd*    */
 
 	{"c0d",	DAC960_MAJOR,				32,	'0' },	/* 48:  c0d0-c0d31 */
 	{"c1d",	DAC960_MAJOR + 1,			32,	'0' },	/* 49:  c1d0-c1d31 */
@@ -772,12 +779,14 @@ gkrellm_sys_proc_init(void)
 #include "../inet.h"
 
 #define	PROC_NET_TCP_FILE	"/proc/net/tcp"
+#define	PROC_NET_UDP_FILE	"/proc/net/udp"
 #if defined(INET6)
 #define	PROC_NET_TCP6_FILE	"/proc/net/tcp6"
+#define	PROC_NET_UDP6_FILE	"/proc/net/udp6"
 #endif
 
-void
-gkrellm_sys_inet_read_tcp_data(void)
+static void
+inet_read_data(gchar *fname, gboolean is_udp)
 	{
 	FILE		*f;
 	ActiveTCP	tcp;
@@ -785,10 +794,9 @@ gkrellm_sys_inet_read_tcp_data(void)
 	gint		tcp_status;
 	gulong		addr;
 
-	if ((f = fopen(PROC_NET_TCP_FILE, "r")) != NULL)
+	if ((f = fopen(fname, "r")) != NULL)
 		{
 		fgets(buf, sizeof(buf), f);		/* header */
-
 		while (fgets(buf, sizeof(buf), f))
 			{
 			sscanf(buf, "%*d: %*x:%x %lx:%x %x", &tcp.local_port,
@@ -797,14 +805,25 @@ gkrellm_sys_inet_read_tcp_data(void)
 			tcp.family = AF_INET;
 			if (tcp_status != TCP_ALIVE)
 				continue;
+			tcp.is_udp = is_udp;
 			gkrellm_inet_log_tcp_port_data(&tcp);
 			}
 		fclose(f);
 		}
+	}
+
 #if defined(INET6)
-	if ((f = fopen(PROC_NET_TCP6_FILE, "r")) != NULL)
+static void
+inet6_read_data(gchar *fname, gboolean is_udp)
+	{
+	FILE		*f;
+	ActiveTCP	tcp;
+	gchar		buf[512];
+	gint		tcp_status;
+
+	if ((f = fopen(fname, "r")) != NULL)
 		{
-		fgets(buf, sizeof(buf), f);		/* Waste first line */
+		fgets(buf, sizeof(buf), f);		/* header */
 		while (fgets(buf, sizeof(buf), f))
 			{
 			sscanf(buf, "%*d: %*x:%x %8x%8x%8x%8x:%x %x",
@@ -817,10 +836,22 @@ gkrellm_sys_inet_read_tcp_data(void)
 			tcp.family = AF_INET6;
 			if (tcp_status != TCP_ALIVE)
 				continue;
+			tcp.is_udp = is_udp;
 			gkrellm_inet_log_tcp_port_data(&tcp);
 			}
 		fclose(f);
 		}
+	}
+#endif	/* INET6 */
+
+void
+gkrellm_sys_inet_read_tcp_data(void)
+	{
+	inet_read_data(PROC_NET_TCP_FILE, FALSE);
+	inet_read_data(PROC_NET_UDP_FILE, TRUE);
+#if defined(INET6)
+	inet6_read_data(PROC_NET_TCP6_FILE, FALSE);
+	inet6_read_data(PROC_NET_UDP6_FILE, TRUE);
 #endif	/* INET6 */
 	}
 
@@ -960,7 +991,7 @@ gkrellm_sys_net_read_data(void)
 				|  If so, just using the 32 low bits of a long long is OK.
 				*/
 				if (   rx == ULONG_MAX && errno == ERANGE
-					&& sscanf(s, "%Ld", &ll) == 1
+					&& sscanf(s, "%" PRIu64, &ll) == 1
 				   )
 					rx = (gulong) ll;
 				}
@@ -968,7 +999,7 @@ gkrellm_sys_net_read_data(void)
 				{
 				tx = strtoul(s, NULL, 0);
 				if (   tx == ULONG_MAX && errno == ERANGE
-					&& sscanf(s, "%Ld", &ll) == 1
+					&& sscanf(s, "%" PRIu64, &ll) == 1
 				   )
 					tx = (gulong) ll;
 				}
@@ -1074,7 +1105,7 @@ get_io_indices(void)
 		fclose(f);
 		}
 	if (_GK.debug_level & DEBUG_NET)
-		printf(_("rx_bytes=%d tx_bytes=%d rx_packets=%d tx_packets=%d\n"),
+		printf("rx_bytes=%d tx_bytes=%d rx_packets=%d tx_packets=%d\n",
 				rx_bytes_index, tx_bytes_index,
 				rx_packets_index, tx_packets_index);
 	}
@@ -1104,7 +1135,7 @@ static guint64	swap_total, swap_used;
 static void
 tagged_format_meminfo(gchar *buf, guint64 *mint)
 	{
-	sscanf(buf,"%Lu", mint);
+	sscanf(buf,"%" PRIu64, mint);
 	*mint *= 1024;
 	}
 
@@ -1128,7 +1159,8 @@ gkrellm_sys_mem_read_data(void)
 		if (buf[0] == 'M')
 			{
 			if (!strncmp(buf, "Mem:", 4))
-				sscanf(buf + 5, "%Lu %Lu %Lu %Lu %Lu %Lu",
+				sscanf(buf + 5,
+		"%" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64,
 						&total, &x_used, &free,
 						&shared, &buffers, &cached);
 			else if (!strncmp(buf, "MemTotal:", 9))
@@ -1144,7 +1176,8 @@ gkrellm_sys_mem_read_data(void)
 		else if (buf[0] == 'S')
 			{
 			if (!strncmp(buf, "Swap:", 5))
-				sscanf(buf + 6,"%Lu %Lu", &swap_total, &swap_used);
+				sscanf(buf + 6,"%" PRIu64 " %" PRIu64,
+						&swap_total, &swap_used);
 			else if (!strncmp(buf, "SwapTotal:", 10))
 				tagged_format_meminfo(buf + 11, &swap_total);
 			else if (!strncmp(buf, "SwapFree:", 9))
@@ -1619,7 +1652,7 @@ acpi_battery_data(BatteryFile *bf)
 		{
 		/*	present:			{yes|no}
 		|	capacity state:		ok
-		|	charging state:		{charging|discharging|unknown}
+		|	charging state:		{charged|charging|discharging|unknown}
 		|	present rate:		15000 mW
 		|	remaining capacity:	31282 mWh
 		|	present voltage:	16652 mV
@@ -1629,6 +1662,7 @@ acpi_battery_data(BatteryFile *bf)
 			have_charging_state = TRUE;
 			if (   (!strcmp(s1, "unknown") && on_line)
 				|| !strcmp(s1, "charging")
+				|| !strcmp(s1, "charged")
 			   )
 				charging = TRUE;
 			continue;
@@ -1900,9 +1934,10 @@ gkrellm_sys_uptime_init(void)
 #define	THERMAL_INTERFACE			2
 #define	THERMAL_ZONE_INTERFACE		3
 #define	NVIDIA_SETTINGS_INTERFACE	4
-#define IBM_ACPI_INTERFACE			5
-#define UNINORTH_INTERFACE			6
-#define WINDFARM_INTERFACE			7
+#define	NVCLOCK_INTERFACE			5
+#define IBM_ACPI_INTERFACE			6
+#define UNINORTH_INTERFACE			7
+#define WINDFARM_INTERFACE			8
 
 #define IBM_ACPI_FAN_FILE	"/proc/acpi/ibm/fan"
 #define IBM_ACPI_THERMAL	"/proc/acpi/ibm/thermal"
@@ -2014,20 +2049,45 @@ sensors_config_migrate(gchar *current_name, gchar *config_name,
 	return FALSE;
 	}
 
+static gboolean
+check_voltage_name(const gchar *name, gint *id)
+	{
+	gint		i = 0,
+				len;
+	static gint	id_default;
+
+	if (!name)
+		return FALSE;
+	len = strlen(name);
+	if (!strncmp(name, "in", 2) && isdigit(name[2]))		/* inX */
+		i = atoi(name + 2);
+	else if (!strncmp(name, "vi", 2) && isdigit(name[3]))	/* vidX/vinX */
+		i = atoi(name + 3);
+	else if (!strcmp(name, "vdd"))
+		i = 0;
+	else if (isdigit(name[0]) && name[len - 1] == 'V')	/* via686a 2.0V etc */
+		i = id_default++;
+	else
+		return FALSE;
+	if (id)
+		*id = i;
+	return TRUE;
+	}
+
 #ifdef HAVE_LIBSENSORS
 #include <sensors/sensors.h>
 
 static gboolean
 libsensors_init(void)
 	{
-	gint			nr, nr1, nr2, len;
-	const sensors_chip_name		*name;
-	const sensors_feature_data	*feature;
-	gchar			*label, *s;
-	FILE			*f;
+	gint			nr, len, iodev;
+	const sensors_chip_name	*name;
+	gchar			*label, *busname, *s;
 	gchar			id_name[512], sensor_path[512];
 	gint			type, n_sensors_added;
 	ConfigMap		*config_map;
+#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
+	FILE			*f;
 
 	f = fopen("/etc/sensors.conf", "r");
 	if (!f)
@@ -2052,7 +2112,8 @@ libsensors_init(void)
 	nr = 0;
 	while ((name = sensors_get_detected_chips(&nr)))
 		{
-		nr1 = 0, nr2 = 0;
+		const sensors_feature_data	*feature;
+		gint nr1 = 0, nr2 = 0;
 		while ((feature = sensors_get_all_features(*name, &nr1, &nr2)))
 			{
 			if (   sensors_get_ignored(*name, feature->number)
@@ -2075,7 +2136,7 @@ libsensors_init(void)
 					type = SENSOR_TEMPERATURE;
 				else if (!strncmp(feature->name, "fan", 3))
 					type = SENSOR_FAN;
-				else if (!strncmp(feature->name, "in", 2))
+				else if (check_voltage_name(feature->name, NULL))
 					type = SENSOR_VOLTAGE;
 				else
 					{
@@ -2102,6 +2163,11 @@ libsensors_init(void)
 					continue;
 					}
 
+				/* notice that we store the bus and addr both in iodev as
+				|  2 _signed 16 bit ints. */
+				iodev = (name->bus & 0xFFFF) | (name->addr << 16);
+				busname = name->busname;
+				
 				if (sensors_get_label(*name, feature->number, &label) != 0)
 					{
 					if (_GK.debug_level & DEBUG_SENSORS)
@@ -2109,7 +2175,119 @@ libsensors_init(void)
 								id_name);
 					label = NULL;
 					}
+#else /* libsensors4 code */
+	if (sensors_init(NULL) != 0)
+		{
+		if (_GK.debug_level & DEBUG_SENSORS)
+			printf("libsensors: init failed!\n");
+		return FALSE;
+		}
 
+	if (_GK.debug_level & DEBUG_SENSORS)
+		printf("libsensors: init OK\n");
+
+	n_sensors_added = 0;
+	nr = 0;
+	while ((name = sensors_get_detected_chips(NULL, &nr)))
+		{
+		const sensors_subfeature *feature;
+		const sensors_feature *main_feature;
+		gint nr1 = 0;
+		while ((main_feature = sensors_get_features(name, &nr1)))
+			{
+			switch (name->bus.type)
+			  {
+			  case SENSORS_BUS_TYPE_I2C:
+			  case SENSORS_BUS_TYPE_SPI:
+				snprintf(id_name, sizeof(id_name), "%s@%d:%x/%s",
+					name->prefix, name->bus.nr, name->addr, main_feature->name);
+				break;
+			  default:
+				snprintf(id_name, sizeof(id_name), "%s@%x/%s",
+					name->prefix, name->addr, main_feature->name);
+			          
+			  }
+			/* We need to store both the prefix and the path, but we
+			|  only have one string, so concat them together separated by :
+			*/
+			snprintf(sensor_path, sizeof (sensor_path), "%s:%s",
+				name->prefix, name->path ? name->path : "NULL");
+
+			switch (main_feature->type)
+			  {
+			  case SENSORS_FEATURE_IN:
+				type = SENSOR_VOLTAGE;
+				feature = sensors_get_subfeature(name, 
+						main_feature, SENSORS_SUBFEATURE_IN_INPUT);
+			  	break;
+			  case SENSORS_FEATURE_FAN:
+				type = SENSOR_FAN;
+				feature = sensors_get_subfeature(name, 
+						main_feature, SENSORS_SUBFEATURE_FAN_INPUT);
+				break;
+			  case SENSORS_FEATURE_TEMP:
+				type = SENSOR_TEMPERATURE;
+				feature = sensors_get_subfeature(name, 
+						main_feature, SENSORS_SUBFEATURE_TEMP_INPUT);
+				break;
+			  default:
+				if (_GK.debug_level & DEBUG_SENSORS)
+					printf("libsensors: error determining type for: %s\n",
+							id_name);
+				continue;
+			  }
+
+			if (!feature)
+				{
+				if (_GK.debug_level & DEBUG_SENSORS)
+					printf("libsensors: error could not get input subfeature for: %s\n",
+							id_name);
+				continue;
+				}
+
+			/* failsafe tests, will bus type and nr fit in 8 bits
+			   signed and addr fit in 16 bits signed ?
+			*/
+			if (name->bus.type != ((name->bus.type << 24) >> 24))
+				{
+				if (_GK.debug_level & DEBUG_SENSORS)
+					printf("libsensors: bus-type bigger than 8 bits: %s\n",
+							id_name);
+				continue;
+				}
+			if (name->bus.nr != ((name->bus.nr << 24) >> 24))
+				{
+				if (_GK.debug_level & DEBUG_SENSORS)
+					printf("libsensors: bus-nr bigger than 8 bits: %s\n",
+							id_name);
+				continue;
+				}
+			if (name->addr != ((name->addr << 16) >> 16))
+				{
+				if (_GK.debug_level & DEBUG_SENSORS)
+					printf("libsensors: addr bigger than 16 bits: %s\n",
+							id_name);
+				continue;
+				}
+
+			/* notice that we store the bus id, type and addr in
+			   iodev as 2 signed 8 bit ints and one 16 bit int */
+			iodev = (name->bus.type & 0xFF) |
+				((name->bus.nr & 0xFF) << 8) |
+				(name->addr << 16);
+			busname = name->path;
+
+			label = sensors_get_label(name, main_feature);
+			if (!label && (_GK.debug_level & DEBUG_SENSORS))
+				printf("libsensors: error getting label for: %s\n",
+						id_name);
+
+			/* additional { to match "if (get_ignored(..) {"
+			   from libsensors3 code */
+				{
+#endif
+				if (label)
+				{
 				/* Strip some common post/prefixes for smaller default labels
 				*/
 				len = strlen(label);
@@ -2136,14 +2314,13 @@ libsensors_init(void)
 						memmove(label, label + 4, strlen (label + 4) + 1);
 					break;
 				  }
-				/* notice that we store the bus and addr both in iodev as
-				|  2 _signed 16 bit ints.  Default factor of zero tells
-				|  sensor.c that sensor formula is handled, ie via
+				}
+				/* Default factor of zero tells sensor.c
+				|  that sensor formula is handled, ie via
 				|  /etc/sensors.conf.
 				*/
 				gkrellm_sensors_add_sensor(type, sensor_path, id_name,
-							feature->number,
-							(name->bus & 0xFFFF) | (name->addr << 16),
+							feature->number, iodev,
 							LIBSENSORS_INTERFACE, 0.0, 0.0,
 							NULL, label);
 				++n_sensors_added;
@@ -2153,10 +2330,10 @@ libsensors_init(void)
 				if (_GK.debug_level & DEBUG_SENSORS)
 					printf("%s %s %x\n",
 							sensor_path, id_name,
-							(name->bus & 0xFFFF) | (name->addr << 16));
+							iodev);
 
-				if (   name->busname
-				    && (s = strrchr(name->busname, '/')) != NULL
+				if (   busname
+				    && (s = strrchr(busname, '/')) != NULL
 				    && *(s + 1)
 				   )
 					{
@@ -2194,8 +2371,9 @@ libsensors_get_value(char *sensor_path, int id, int iodev, float *value)
 		}
 	*p = 0;						/* We must undo this !! (or make a copy) */
 	name.prefix = sensor_path;
-	name.bus = (iodev << 16) >> 16;	/* sign extend the low 16 bits */
 	name.addr = iodev >> 16;
+#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
+	name.bus = (iodev << 16) >> 16;	/* sign extend the low 16 bits */
 	name.busname = p + 1;
 	if (!strcmp(name.busname, "NULL"))
 		name.busname = NULL;
@@ -2213,6 +2391,32 @@ libsensors_get_value(char *sensor_path, int id, int iodev, float *value)
 					name.prefix, name.addr, id);
 		}
 
+#else /* libsensors4 code */
+	name.bus.type = (iodev << 24) >> 24; /* sign extend the low 8 bits */
+	name.bus.nr   = (iodev << 16) >> 24; /* sign extend the 2nd byte */
+	name.path = p + 1;
+	if (!strcmp(name.path, "NULL"))
+		name.path = NULL;
+
+	result = sensors_get_value(&name, id, &val) == 0;
+	
+	if (!result && (_GK.debug_level & DEBUG_SENSORS))
+		{
+			switch (name.bus.type)
+			  {
+			  case SENSORS_BUS_TYPE_I2C:
+			  case SENSORS_BUS_TYPE_SPI:
+				printf(
+					"libsensors: error getting value for: %s@%d:%x feature: %d\n",
+					name.prefix, (int)name.bus.nr, name.addr, id);
+				break;
+			  default:
+				printf("libsensors: error getting value for: %s@%x feature: %d\n",
+					name.prefix, name.addr, id);
+			  }
+		}
+#endif
+
 	if (value)
 		*value = val;
 
@@ -2469,8 +2673,8 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 	if (interface == NVIDIA_SETTINGS_INTERFACE)
 		{
 #if GLIB_CHECK_VERSION(2,0,0)
-		char *args[] = { "nvidia-settings", "-q", sensor_path, NULL };
-		char *output = NULL;
+		gchar	*args[] = { "nvidia-settings", "-q", sensor_path, NULL };
+		gchar	*output = NULL;
 
 		result = g_spawn_sync(NULL, args, NULL,
 					G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
@@ -2479,7 +2683,9 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 		if(result && output)
 			{
 			gfloat dummy;
-			if(!temp) temp = &dummy;
+
+			if(!temp)
+				temp = &dummy;
 			result = (sscanf(output, " Attribute %*s %*s %f", temp) == 1);
 			}
 
@@ -2490,6 +2696,37 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 #endif
 		}
 
+	if (interface == NVCLOCK_INTERFACE)
+		{
+#if GLIB_CHECK_VERSION(2,0,0)
+		gchar	*args[] = { "nvclock", "-T", "-c", sensor_path, NULL };
+		gchar	*output = NULL;
+		gchar	*s = NULL;
+
+		result = g_spawn_sync(NULL, args, NULL,
+					G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+					NULL, NULL, &output, NULL, NULL, NULL);
+
+		if(result && output)
+			{
+			gfloat dummy;
+
+			if(!temp)
+				temp = &dummy;
+			s = strstr(output, "GPU");
+			if (s)
+				result = (sscanf(s, "GPU temperature:%f", temp) == 1);
+			else
+				result = FALSE;
+			}
+
+		g_free(output);
+		return result;
+#else
+		return FALSE;
+#endif
+		}
+
 	if (interface == UNINORTH_INTERFACE || interface == WINDFARM_INTERFACE)
 		{
 		if ((f = fopen(sensor_path, "r")))
@@ -2886,6 +3123,85 @@ sysfs_sensors_init(void)
 	g_dir_close(dir);
 	}
 
+static gint
+sensors_nvidia_settings_ngpus(void)
+	{
+	gint		n = 0;
+#if GLIB_CHECK_VERSION(2,0,0)
+	gchar		*args[] = { "nvidia-settings", "-q", "gpus", NULL };
+	gchar		*output = NULL;
+	gboolean	result;
+
+	result = g_spawn_sync(NULL, args, NULL,
+				G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+				NULL, NULL, &output, NULL, NULL, NULL);
+
+	if(result && output)
+		sscanf(output, "%d", &n);
+	g_free(output);
+#endif
+	if (_GK.debug_level & DEBUG_SENSORS)
+		printf("nvidia-settings gpus = %d\n", n);
+	return n;
+	}
+
+static gint
+sensors_nvclock_ngpus(void)
+	{
+	gint		n = 0;
+#if GLIB_CHECK_VERSION(2,0,0)
+	gchar		*args[] = { "nvclock", "-s", NULL };
+	gchar		*output = NULL, *s;
+	gboolean	result;
+
+	result = g_spawn_sync(NULL, args, NULL,
+				G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+				NULL, NULL, &output, NULL, NULL, NULL);
+
+	if(result && output)
+		{
+		s = g_strrstr(output, "Card number:");
+		if (s)
+			sscanf(s, "Card number: %d", &n);
+		}
+	g_free(output);
+#endif
+	if (_GK.debug_level & DEBUG_SENSORS)
+		printf("nvclock gpus = %d\n", n);
+	return n;
+	}
+
+static void
+sensors_nvclock_init(gboolean enable)
+	{
+	gint	cnt, id;
+	gchar	*sensor_path, *default_label;
+	gchar	id_name[128];
+
+	if (!enable)
+		return;
+	cnt = sensors_nvclock_ngpus();
+	for (id = 1; id <= cnt; ++id)
+		{
+		sensor_path = g_strdup_printf("%d", id);
+		if (gkrellm_sys_sensors_get_temperature(sensor_path, id, 0,
+						NVCLOCK_INTERFACE, NULL))
+			{
+			snprintf(id_name, sizeof(id_name), "nvclock GPU:%d Core", id);
+			if (cnt == 1)
+				default_label = g_strdup_printf("GPU C");
+			else
+				default_label = g_strdup_printf("GPU:%d", id);
+			gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+						sensor_path, id_name,
+						id, 0, NVCLOCK_INTERFACE,
+						1.0, 0.0, NULL, default_label);
+			g_free(default_label);
+			}
+		g_free(sensor_path);
+		}
+	}
+
   /* Remove embedded "-i2c-" or "-isa-" from /proc lm_sensors chip names so
   |  there can be a chance for config name /proc -> /sysfs compatibility.
   |  /proc was used in older 2.4 kerneles.
@@ -2927,7 +3243,7 @@ gkrellm_sys_sensors_init(void)
 	GDir			*dir, *chip_dir;
 	VoltDefault		*voltdefault;
 	gint			id = 0;
-	gint			type, voltdefaultsize, cnt;
+	gint			type, voltdefaultsize, cnt, ngpus_added;
 	gfloat			factor, offset;
 	gchar			*name, *chip_name, *fixed_chip_name, *path, *default_label;
 	gchar			*vref, *sensor_path, *sensor_name, id_name[128], pbuf[128];
@@ -3017,9 +3333,8 @@ gkrellm_sys_sensors_init(void)
 		fclose(f);
 		}
 
-	/* 
-	 * IBM ACPI Cooling Fan Sensor
-	 */
+	/* IBM ACPI Cooling Fan Sensor
+	*/
 	if ((f = fopen(IBM_ACPI_FAN_FILE, "r")) != NULL)
 		{
 		gkrellm_sensors_add_sensor(SENSOR_FAN,
@@ -3028,15 +3343,56 @@ gkrellm_sys_sensors_init(void)
 				1.0, 0.0, NULL, "Fan");
 		fclose(f);
 		}
-	
-	if (gkrellm_sys_sensors_get_temperature("GPUCoreTemp", id, 0,
-					NVIDIA_SETTINGS_INTERFACE, NULL))
+
+	/* nvidia-settings GPU core & ambient temperatues
+	*/
+	cnt = sensors_nvidia_settings_ngpus();
+	ngpus_added = 0;
+	if (cnt < 2)
 		{
-		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
-					"GPUCoreTemp", "nVidia GPU Core",
-					id, 0, NVIDIA_SETTINGS_INTERFACE,
-					1.0, 0.0, NULL, "GPU C");
+		if (gkrellm_sys_sensors_get_temperature("GPUCoreTemp", id, 0,
+						NVIDIA_SETTINGS_INTERFACE, NULL))
+			{
+			gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+						"GPUCoreTemp", "nVidia GPU Core",
+						id, 0, NVIDIA_SETTINGS_INTERFACE,
+						1.0, 0.0, NULL, "GPU C");
+			++ngpus_added;
+			}
 		}
+	else
+		{
+		for (id = 0; id < cnt; ++id)
+			{
+			sensor_path = g_strdup_printf("[gpu:%d]/GPUCoreTemp", id);
+			if (gkrellm_sys_sensors_get_temperature(sensor_path, id, 0,
+							NVIDIA_SETTINGS_INTERFACE, NULL))
+				{
+				snprintf(id_name, sizeof(id_name), "nVidia GPU:%d Core", id);
+				default_label = g_strdup_printf("GPU:%d", id);
+				gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+							sensor_path, id_name,
+							id, 0, NVIDIA_SETTINGS_INTERFACE,
+							1.0, 0.0, NULL, default_label);
+				g_free(default_label);
+				++ngpus_added;
+				}
+			g_free(sensor_path);
+			}
+		}
+
+	/* nvclock can get temps for some cards where nvidia-settings fails.
+	|  nvclock card numbers start at 1.  Make an option because nvclock
+	|  doesn't handle all nvidia chips (6150) where it will always reread
+	|  GPU bios which slows gkrellm startup to several seconds.
+	*/
+	if (ngpus_added == 0)
+		gkrellm_sensors_sysdep_option("use_nvclock",
+			_("Use nvclock for NVIDIA GPU temperatures"),
+			sensors_nvclock_init);
+
+	id = 0;
+	/* Try for ambient only for gpu:0 for now */
 	if (gkrellm_sys_sensors_get_temperature("GPUAmbientTemp", id, 0,
 				NVIDIA_SETTINGS_INTERFACE, NULL))
 		{
@@ -3046,33 +3402,42 @@ gkrellm_sys_sensors_init(void)
 					1.0, 0.0, NULL, "GPU A");
 		}
 
-    if ((dir = g_dir_open(UNINORTH_DIR, 0, NULL)) != NULL)
-        {
-        while (( sensor_name = (gchar *) g_dir_read_name(dir)) != NULL )
-            {
-            if (strncmp(sensor_name, "sensor", 6) == 0)
-                {
-                sensor_path = g_build_filename(UNINORTH_DIR, sensor_name, NULL);
-                if (strncmp(sensor_name + 8, "temperature", 11) == 0)
-                    {
-                    // linux <= 2.6.12 had [cg]pu_temperature instead of
-                    // sensor[12]_temperature
+
+	/* UNINORTH sensors
+	*/
+	if ((dir = g_dir_open(UNINORTH_DIR, 0, NULL)) != NULL)
+		{
+		while (( sensor_name = (gchar *) g_dir_read_name(dir)) != NULL )
+			{
+			if (strncmp(sensor_name, "sensor", 6) == 0)
+				{
+				sensor_path = g_build_filename(UNINORTH_DIR, sensor_name,NULL);
+				if (strncmp(sensor_name + 8, "temperature", 11) == 0)
+					{
+					// linux <= 2.6.12 had [cg]pu_temperature instead of
+					// sensor[12]_temperature
                     
-                    // TODO: read sensorN_location instead of fixed positions
-                    if (sensor_name[6] == '1' || (strncmp(sensor_name, "cpu", 3)) == 0)
-                        gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE, sensor_path,
-                                "cpu topside", id, 0, UNINORTH_INTERFACE,
-                                1.0, 0.0, NULL, "CPU");
-                    else if (sensor_name[6] == '2' || (strncmp(sensor_name, "gpu", 3)) == 0) 
-                        gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE, sensor_path,
-                                "gpu on die", id, 0, UNINORTH_INTERFACE,
-                                1.0, 0.0, NULL, "GPU");
-                    }
-                g_free(sensor_path);
-                }
-            }
-        g_dir_close(dir);
-        }
+					// TODO: read sensorN_location instead of fixed positions
+					if (   sensor_name[6] == '1'
+					    || (strncmp(sensor_name, "cpu", 3)) == 0
+					   )
+						gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+								sensor_path, "cpu topside",
+								id, 0, UNINORTH_INTERFACE,
+								1.0, 0.0, NULL, "CPU");
+					else if (   sensor_name[6] == '2'
+					         || (strncmp(sensor_name, "gpu", 3)) == 0
+					        ) 
+						gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+								sensor_path, "gpu on die",
+								id, 0, UNINORTH_INTERFACE,
+								1.0, 0.0, NULL, "GPU");
+					}
+				g_free(sensor_path);
+				}
+			}
+		g_dir_close(dir);
+		}
 
 	/* Windfarm (PowerMac G5, others?)) */
 	cnt = 0;
@@ -3226,21 +3591,8 @@ gkrellm_sys_sensors_init(void)
 						)
 					)
 				type = SENSOR_FAN;
-			else if (!strncmp(sensor_name, "in", 2) && isdigit(sensor_name[2]))
-				{
+			else if (check_voltage_name(sensor_name, &id))
 				type = SENSOR_VOLTAGE;
-				id = sensor_name[2] - '0';
-				}
-			else if (!strncmp(sensor_name, "vi", 2) && isdigit(sensor_name[3]))
-				{
-				type = SENSOR_VOLTAGE;
-				id = sensor_name[3] - '0';
-				}
-			else if (!strcmp(sensor_name, "vdd"))
-				{
-				type = SENSOR_VOLTAGE;
-				id = 0;
-				}
 			else
 				continue;
 
